import * as THREE from 'three'
import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls.js'
import { VRButton } from 'three/examples/jsm/Addons.js'
import { XRControllerModelFactory } from 'three/examples/jsm/Addons.js'
/**
 * Base
 */
// Canvas
const canvas = document.querySelector('canvas.webgl')

// Scene
const scene = new THREE.Scene()
scene.background = new THREE.Color(0xf0efea)


/**
 * Objects
 */
const floorGeometry = new THREE.PlaneGeometry(5, 20)
const floorMaterial = new THREE.MeshToonMaterial({ color: 0xc0b2b5, side: THREE.DoubleSide })
const floor = new THREE.Mesh(floorGeometry, floorMaterial)
floor.rotation.x = Math.PI * 0.5
scene.add(floor)

// Box
const boxGeometry = new THREE.BoxGeometry(2, 2, 2)
const boxMaterial = new THREE.MeshToonMaterial({ color: 0xfa6e59 })
// Sphere
const sphereGeometry = new THREE.SphereGeometry(1)
const sphereMaterial = new THREE.MeshToonMaterial({ color: 0x4897d8 })
// Torus
const torusGeometry = new THREE.TorusGeometry( 1, 0.25, 9, 100 )
const torusMaterial = new THREE.MeshToonMaterial( { color: 0xf8a055 } )

const spread = 40 // Abstand zwischen den Objekten
const offset = 20 // Verschiebung der Objekte zum Mittelpunkt

// Objekte werden erstellt
for (let i = 0; i < 40; i++) {
    // Box
    const box = new THREE.Mesh(boxGeometry, boxMaterial)
    box.position.set(Math.random() * spread - offset, Math.random() * spread -offset, Math.random() * spread - offset)
    box.rotation.set(Math.random() * Math.PI, Math.random() * Math.PI, Math.random() * Math.PI)
    scene.add(box)

    // Sphere
    const sphere = new THREE.Mesh(sphereGeometry, sphereMaterial)
    sphere.position.set(Math.random() * spread - offset, Math.random() * spread -offset, Math.random() * spread - offset)
    scene.add(sphere)

    // Torus
    const torus = new THREE.Mesh(torusGeometry, torusMaterial)
    torus.position.set(Math.random() * spread - offset, Math.random() * spread -offset, Math.random() * spread - offset)
    torus.rotation.set(Math.random() * Math.PI, Math.random() * Math.PI, Math.random() * Math.PI)
    scene.add(torus)
}


/**
 * Light
 */
const light = new THREE.DirectionalLight( 0xfffff0, 5 )
light.position.set( 0, 2, -5 )
light.castShadow = true
scene.add( light )


/**
 * Sizes
 */
const sizes = {
    width: window.innerWidth,
    height: window.innerHeight
}

window.addEventListener('resize', () =>
{
    // Update sizes
    sizes.width = window.innerWidth
    sizes.height = window.innerHeight

    // Update camera
    camera.aspect = sizes.width / sizes.height
    camera.updateProjectionMatrix()

    // Update renderer
    renderer.setSize(sizes.width, sizes.height)
    renderer.setPixelRatio(Math.min(window.devicePixelRatio, 2))
})


/**
 * Camera
 */
// Base camera
const camera = new THREE.PerspectiveCamera(75, sizes.width / sizes.height, 0.1, 100)
camera.position.set(3, 2, -20)
scene.add(camera)

// Controls
const controls = new OrbitControls(camera, canvas)
controls.enableDamping = true


/**
 * Renderer
 */
const renderer = new THREE.WebGLRenderer({
    canvas: canvas
})
renderer.setSize(sizes.width, sizes.height)
renderer.setPixelRatio(Math.min(window.devicePixelRatio, 2))


/**
 * VR
*/
// VR aktivieren
renderer.xr.enabled = true

// VRButton hinzufügen
document.body.appendChild(VRButton.createButton(renderer))

// Session prüfen
renderer.xr.addEventListener('sessionstart', () => {
    console.log('session started')
})


// Controller 1 erstellen
let controller1 = renderer.xr.getController(0)

controller1.addEventListener('selectstart', () => {
    console.log('selectstart')
})
controller1.addEventListener('selectend', () => {
    console.log('selectend')
})
controller1.addEventListener('connected', (event) => {
    console.log('connected', event)
})
controller1.addEventListener('disconnected', () => {
    this.remove(this.children[0])
    console.log('disconnected')
})

scene.add(controller1)


// ControllerGrip1 erstellen
const controllerModelFactory = new XRControllerModelFactory()

let controllerGrip1 = renderer.xr.getControllerGrip(0)
controllerGrip1.add(controllerModelFactory.createControllerModel(controllerGrip1))

scene.add(controllerGrip1)


// Controller 2 erstellen
let controller2 = renderer.xr.getController(1)

controller2.addEventListener('selectstart', () => {
    console.log('selectstart')
})
controller2.addEventListener('selectend', () => {
    console.log('selectend')
})
controller2.addEventListener('connected', (event) => {
    console.log('connected', event)
})
controller2.addEventListener('disconnected', () => {
    this.remove(this.children[1])
    console.log('disconnected')
})

scene.add(controller2)


// ControllerGrip2 erstellen

let controllerGrip2 = renderer.xr.getControllerGrip(1)
controllerGrip2.add(controllerModelFactory.createControllerModel(controllerGrip2))

scene.add(controllerGrip2)


/**
 * Animate
 */
const clock = new THREE.Clock()

const animate = () =>
{
    const elapsedTime = clock.getElapsedTime()

    // Controls update
    controls.update()

    // Render
    renderer.render(scene, camera)

    // Tick wird auf nächstem Frame wieder aufgerufen
}

renderer.setAnimationLoop(animate)

